@@ -5,6 +5,10 @@ use Getopt::Long;
 use Carp;
 use Test::More;
 use IO::Socket::INET;
+use Sys::Hostname;
+use constant LOCK_EX => 2;
+use constant LOCK_NB => 4;
+use File::Spec;
 
 use vars qw($VERSION @ISA);
 
@@ -16,11 +20,11 @@ App::Prove::Plugin::Distributed - to distribute test job using client and server
 
 =head1 VERSION
 
-Version 0.01
+Version 0.02
 
 =cut
 
-$VERSION = '0.01';
+$VERSION = '0.02';
 
 =head3 C<load>
 
@@ -38,7 +42,7 @@ sub load {
     {
         local @ARGV = @args;
 
-        push @ARGV, grep {/^--/} @{ $app->{argv} };
+        push @ARGV, grep { /^--/ } @{ $app->{argv} };
         $app->{argv} = [ grep { !/^--/ } @{ $app->{argv} } ];
         Getopt::Long::Configure(qw(no_ignore_case bundling pass_through));
 
@@ -48,6 +52,8 @@ sub load {
             'distributed-type=s' => \$app->{distributed_type},
             'start-up=s'         => \$app->{start_up},
             'tear-down=s'        => \$app->{tear_down},
+            'error-log=s'        => \$app->{error_log},
+            'detach'             => \$app->{detach},
         ) or croak('Unable to continue');
     }
     my $type = $app->{distributed_type};
@@ -56,7 +62,8 @@ sub load {
         && $app->{argv}->[0] =~ /$option_name=number_of_workers=(\d+)/ )
     {
         if ( $app->{jobs} ) {
-            die "-j and $option_name=number_of_workers are mutually exclusive.\n";
+            die
+              "-j and $option_name=number_of_workers are mutually exclusive.\n";
         }
         else {
             $app->{jobs} = $1;
@@ -64,10 +71,11 @@ sub load {
     }
     else {
         $app->{jobs} ||= 1;
-        unshift @{ $app->{argv} }, "$option_name=number_of_workers=" . $app->{jobs};
+        unshift @{ $app->{argv} },
+          "$option_name=number_of_workers=" . $app->{jobs};
     }
 
-    for (qw(start_up tear_down)) {
+    for (qw(start_up tear_down error_log detach)) {
         if ( $app->{$_} ) {
             unshift @{ $app->{argv} }, "$option_name=$_=" . $app->{$_};
         }
@@ -77,19 +85,20 @@ sub load {
 
         #LSF: Set the iterator.
         $app->sources(
-            [   'Worker'
-                    . (
+            [
+                'Worker'
+                  . (
                     $app->{distributed_type}
                     ? '::' . $app->{distributed_type}
                     : ''
-                    )
+                  )
             ]
         );
         return 1;
     }
 
     my $original_perl_5_lib = $ENV{PERL5LIB} || '';
-    my @original_include    = @INC;
+    my @original_include = @INC;
     if ( $app->{includes} ) {
         my @includes = split /:/, $original_perl_5_lib;
         unshift @includes, @original_include;
@@ -115,7 +124,13 @@ sub load {
     while (1) {
 
         #LSF: The is the server to serve the test.
-        $class->start_server( $app->{manager} );
+        $class->start_server(
+            app  => $app,
+            spec => $app->{manager},
+            ( $app->{error_log} ? ( error_log => $app->{error_log} ) : () ),
+            ( $app->{detach}    ? ( detach    => $app->{detach} )    : () ),
+
+        );
     }
 
     #LSF: Anything below here might not be called.
@@ -140,7 +155,9 @@ Parameter is the contoller peer address.
 
 sub start_server {
     my $class = shift;
-    my $spec  = shift;
+    my %args  = @_;
+    my ( $app, $spec, $error_log, $detach ) =
+      @args{ 'app', 'spec', 'error_log', 'detach' };
 
     my $socket = IO::Socket::INET->new(
         PeerAddr => $spec,
@@ -166,14 +183,48 @@ sub start_server {
         $builder->output($socket);
         $builder->failure_output($socket);
         $builder->todo_output($socket);
-	*STDERR = $socket;
-	*STDOUT = $socket;
+        *STDERR = $socket;
+        *STDOUT = $socket;
+        if ($detach) {
+            my $command = join ' ',
+              ( $job_info,
+                ( $app->{test_args} ? @{ $app->{test_args} } : () ) );
+            {
+                require TAP::Parser::Source;
+                require TAP::Parser::SourceHandler::Worker;
+                my $source = TAP::Parser::Source->new();
+                $source->raw( \$job_info )->assemble_meta;
+                my $vote =
+                  TAP::Parser::SourceHandler::Worker->can_handle($source);
+                if ( $vote >= 0.75 ) {
+                    $command = $^X . ' ' . $command;
+                }
+                print $socket `$command`;
+            }
+            exit;
+        }
         unless ( $class->_do($job_info) ) {
             print $socket "$0\n$error\n\b";
-	    use IO::File;
-	    my $fh = IO::File->new($$, 'w');
-	    print $fh "$0\n$error\n\b";
-	    close $fh;
+            if ($error_log) {
+                use IO::File;
+                my $fh = IO::File->new( "$error_log", 'a+' );
+                unless ( flock( $fh, LOCK_EX | LOCK_NB ) ) {
+                    warn
+"can't immediately write-lock the file ($!), blocking ...";
+                    unless ( flock( $fh, LOCK_EX ) ) {
+                        die "can't get write-lock on numfile: $!";
+                    }
+                }
+                my $server_spec =
+                  ( $socket->sockhost eq '0.0.0.0'
+                    ? hostname
+                    : $socket->sockhost )
+                  . ':'
+                  . $socket->sockport;
+                print $fh
+"<< START $job_info >>\nSERVER: $server_spec\nPID: $$\nERROR: $error\n<< END $job_info >>\n\b";
+                close $fh;
+            }
         }
         exit;
     }
@@ -188,22 +239,31 @@ sub _do {
     my $proto    = shift;
     my $job_info = shift;
 
+    my $cwd = File::Spec->rel2abs('.');
+
     #LSF: The code from here to exit is from  L<FCGI::Daemon> module.
     local *CORE::GLOBAL::exit = sub { die 'notr3a11yeXit' };
     local $0 = $job_info;    #fixes FindBin (in English $0 means $PROGRAM_NAME)
     no strict;               # default for Perl5
-    do $0;                   # do $0; could be enough for strict scripts
-    if ($EVAL_ERROR) {
-        $EVAL_ERROR =~ s{\n+\z}{};
-        unless ( $EVAL_ERROR =~ m{^notr3a11yeXit} ) {
-            $error = $EVAL_ERROR;
+    {
+
+        package main;
+        local @ARGV = ();
+        do $0;               # do $0; could be enough for strict scripts
+        chdir($cwd);
+
+        if ($EVAL_ERROR) {
+            $EVAL_ERROR =~ s{\n+\z}{};
+            unless ( $EVAL_ERROR =~ m{^notr3a11yeXit} ) {
+                $error = $EVAL_ERROR;
+                return;
+            }
+        }
+        elsif ($@) {
+            $error = $@;
             return;
         }
     }
-    elsif ($@) {
-        $error = $@;
-        return;
-    }
     return 1;
 }
 
